package GGI::Pixel;

=head1 NAME

GGI::Pixel - Perl interface to LibGGI Get/Put buffers and ggi_pixelformat

=head1 SYNOPSIS

  use GGI::Pixel;

  $pixels = new GGI::Pixel $visual;
  $pixels = new GGI::Pixel $pixel;
  $pixels = new GGI::Pixel $visual $numpixels;
  $pixels = new GGI::Pixel $pixel $numpixels;

  $bytes_allocated_per_pixel = $pixels->pixel_bytes;
  $bits_allocated_per_pixel = $pixels->pixel_size;
  $bits_used_per_pixel = $pixels->pixel_depth;

  ($red_mask, $red_shift) = ($pixels->red_mask, $pixels->red_shift);
  ($blue_mask, $blue_shift) = ($pixels->blue_mask, $pixels->blue_shift);
  ($green_mask, $green_shift) = ($pixels->green_mask, $pixels->green_shift);
  ($alpha_mask, $alpha_shift) = ($pixels->alpha_mask, $pixels->alpha_shift);
  ($clut_mask, $clut_shift) = ($pixels->clut_mask, $pixels->clut_shift);
  ($fg_mask, $fg_shift) = ($pixels->fg_mask, $pixels->fg_shift);
  ($bg_mask, $bg_shift) = ($pixels->bg_mask, $pixels->bg_shift);
  ($texture_mask, $texture_shift) = 
    ($pixels->texture_mask, $pixels->texture_shift);

  $flags = $pixels->flags;
  print "is reverse endian" if $flags == GGI::Pixel::PF_REVERSE_ENDIAN;

  $meaning = $pixels->meaning($bitnum);
  print "is a red color bit." if $meaning == GGI::Pixel::BM_SUB_RED;

  $format = $pixels->format;
  $othervis->pixel($x, $y, $pixels) if $format == $otherformat;

  $number_of_pixels = $pixels->size;
  $pixels->size($number_of_pixels);


=head1 DESCRIPTION

The GGI::Pixel object encapsulates raw memory buffers used with
GGI:: objects to copy and manipulate graphic image data, and the
associated ggi_pixelformat structure.

=cut

use strict;
use Carp;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD);

require Exporter;
require DynaLoader;
require AutoLoader;


=head1 Exported constants

  ATTR_BGCOLOR
  ATTR_BLINK
  ATTR_BOLD
  ATTR_BRIGHT
  ATTR_FGCOLOR
  ATTR_FONT
  ATTR_HALF
  ATTR_INTENSITY
  ATTR_ITALIC
  ATTR_NORMAL
  ATTR_REVERSE
  ATTR_UNDERLINE
  BM_SUB_ALPHA
  BM_SUB_BGCOL
  BM_SUB_BLINK
  BM_SUB_BLUE
  BM_SUB_BOLD
  BM_SUB_CLUT
  BM_SUB_CYAN
  BM_SUB_FGCOL
  BM_SUB_FONTSEL
  BM_SUB_GREEN
  BM_SUB_INTENSITY
  BM_SUB_ITALIC
  BM_SUB_K
  BM_SUB_MAGENTA
  BM_SUB_MODESEL
  BM_SUB_PALSEL
  BM_SUB_RED
  BM_SUB_TEXNUM
  BM_SUB_U
  BM_SUB_UNDERLINE
  BM_SUB_V
  BM_SUB_WINDOWID
  BM_SUB_WRITEPROT
  BM_SUB_Y
  BM_SUB_YELLOW
  BM_SUB_ZBUFFER
  BM_TYPE_ATTRIB
  BM_TYPE_COLOR
  BM_TYPE_LOGIC
  BM_TYPE_NONE
  COLOR_PRECISION
  PALETTE_DONTCARE
  PF_EXTENDED
  PF_HAM
  PF_HIGHBIT_RIGHT
  PF_REVERSE_ENDIAN

=cut

@ISA = qw(Exporter DynaLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	ATTR_BGCOLOR
	ATTR_BLINK
	ATTR_BOLD
	ATTR_BRIGHT
	ATTR_FGCOLOR
	ATTR_FONT
	ATTR_HALF
	ATTR_INTENSITY
	ATTR_ITALIC
	ATTR_NORMAL
	ATTR_REVERSE
	ATTR_UNDERLINE
	BM_SUB_ALPHA
	BM_SUB_BGCOL
	BM_SUB_BLINK
	BM_SUB_BLUE
	BM_SUB_BOLD
	BM_SUB_CLUT
	BM_SUB_CYAN
	BM_SUB_FGCOL
	BM_SUB_FONTSEL
	BM_SUB_GREEN
	BM_SUB_INTENSITY
	BM_SUB_ITALIC
	BM_SUB_K
	BM_SUB_MAGENTA
	BM_SUB_MODESEL
	BM_SUB_PALSEL
	BM_SUB_RED
	BM_SUB_TEXNUM
	BM_SUB_U
	BM_SUB_UNDERLINE
	BM_SUB_V
	BM_SUB_WINDOWID
	BM_SUB_WRITEPROT
	BM_SUB_Y
	BM_SUB_YELLOW
	BM_SUB_ZBUFFER
	BM_TYPE_ATTRIB
	BM_TYPE_COLOR
	BM_TYPE_LOGIC
	BM_TYPE_NONE
	COLOR_PRECISION
	PALETTE_DONTCARE
	PF_EXTENDED
	PF_HAM
	PF_HIGHBIT_RIGHT
	PF_REVERSE_ENDIAN
);
$VERSION = '0.03';


=head1 Methods

The methods applicable to GGI::Pixel objects are as follows:

=head2 new

The GGI::Pixel constructor must be provided with either a GGI::Pixel
or a GGI object and optionally a number of pixels to allocate.

=cut

# We want the pixel format to remain after the visual is gone/changed,
# but not to have a ggi_pixelformat structure for each buffer object.
# To do this we implement an internal package to keep reference counts
# on allocated ggi_pixelformat structures.  All it does is
# deallocate the object when the last reference is destroyed.

package GGI::Pixel::_internal;

sub DESTROY {
    my($this) = shift();
    GGI::Pixel::_free($$this);
}

package GGI::Pixel;

sub new {
    my ($class) = shift();
    my $this = {};
    $this->{obj} = shift();
    return undef unless ref($this->{obj});
    if (UNIVERSAL::isa($this->{obj}, 'GGI')) {
	$this->{obj} = $this->{obj}->_get_pixel_format;
    }
    if (UNIVERSAL::isa($this->{obj}, 'GGI::Pixel')) {
	$this->{obj} = $this->{obj}{obj};
    };
    return undef 
	unless ref($this->{obj}) 
	    and UNIVERSAL::isa($this->{obj}, 'GGI::Pixel::_internal');
    $this->{size} = shift();
    return undef if $this->{size} < 0;
    bless $this, $class;
    $this->{buf} = $this->_malloc(${$this->{obj}}, $this->{size})
	if $this->{size} > 0;
    return undef unless ($this->{buf} or not $this->{size});
    return $this;
}

sub DESTROY {
    my ($this) = shift();
    $this->_free_buffer($this->{buf});
}

# These methods do nothing in this class, they are for GGI::AutoPixel

sub resize_put {
}

sub resize_get {
}

sub lastused {
}

=head1 pixel_bytes

Gets the size in bytes of one pixel as represented in the associated 
buffer.

=cut

sub pixel_bytes {
    my ($this) = shift();
    return $this->_get_pixel_bytes(${$this->{obj}});
}

=head1 pixel_size

Gets the number of bits in a single pixel value, some of which may
be unused.

=cut

sub pixel_size {
    my ($this) = shift();
    return $this->_get_pixel_size(${$this->{obj}});
}

=head1 pixel_depth

Gets the number of bits in a single pixel value which are actually used.

=cut

sub pixel_depth {
    my ($this) = shift();
    return $this->_get_pixel_depth(${$this->{obj}});
}

=head1 red_mask

Gets a mask that tells which bits are used to represent the 
red color channel.  See the LibGGI docs.

=cut

sub red_mask {
    my ($this) = shift();
    return $this->_get_red_mask(${$this->{obj}});
}

=head1 red_shift

Gets a shift value that can be used to generate the red channel bits
for a red channel value.  See the LibGGI docs.

=cut

sub red_shift {
    my ($this) = shift();
    return $this->_get_red_shift(${$this->{obj}});
}

=head1 green_mask

Gets a mask that tells which bits are used to represent the 
green color channel.  See the LibGGI docs.

=cut

sub green_mask {
    my ($this) = shift();
    return $this->_get_green_mask(${$this->{obj}});
}

=head1 green_shift

Gets a shift value that can be used to generate the green channel bits
for a green channel value.  See the LibGGI docs.

=cut

sub green_shift {
    my ($this) = shift();
    return $this->_get_green_shift(${$this->{obj}});
}

=head1 blue_mask

Gets a mask that tells which bits are used to represent the 
blue color channel.  See the LibGGI docs.

=cut

sub blue_mask {
    my ($this) = shift();
    return $this->_get_blue_mask(${$this->{obj}});
}

=head1 blue_shift

Gets a shift value that can be used to generate the blue channel bits
for a blue channel value.  See the LibGGI docs.

=cut

sub blue_shift {
    my ($this) = shift();
    return $this->_get_blue_shift(${$this->{obj}});
}

=head1 alpha_mask

Gets a mask that tells which bits are used to represent the 
alpha color channel.  See the LibGGI docs.

=cut

sub alpha_mask {
    my ($this) = shift();
    return $this->_get_alpha_mask(${$this->{obj}});
}

=head1 alpha_shift

Gets a shift value that can be used to generate the alpha channel bits
for a alpha channel value.  See the LibGGI docs.

=cut

sub alpha_shift {
    my ($this) = shift();
    return $this->_get_alpha_shift(${$this->{obj}});
}


=head1 clut_mask

Gets a mask that tells which bits are used to represent the 
clut index.  See the LibGGI docs.

=cut

sub clut_mask {
    my ($this) = shift();
    return $this->_get_clut_mask(${$this->{obj}});
}

=head1 clut_shift

Gets a shift value that can be used to generate the bits
for a clut index.  See the LibGGI docs.

=cut

sub clut_shift {
    my ($this) = shift();
    return $this->_get_clut_shift(${$this->{obj}});
}

=head1 fg_mask

Gets a mask that tells which bits are used to represent the 
fg color value.  See the LibGGI docs.

=cut

sub fg_mask {
    my ($this) = shift();
    return $this->_get_fg_mask(${$this->{obj}});
}

=head1 fg_shift

Gets a shift value that can be used to generate the bits
to represent a fg color value.  See the LibGGI docs.

=cut

sub fg_shift {
    my ($this) = shift();
    return $this->_get_fg_shift(${$this->{obj}});
}

=head1 bg_mask

Gets a mask that tells which bits are used to represent the 
bg color color value.  See the LibGGI docs.

=cut

sub bg_mask {
    my ($this) = shift();
    return $this->_get_bg_mask(${$this->{obj}});
}

=head1 bg_shift

Gets a shift value that can be used to generate the bits
to represent a bg color value.  See the LibGGI docs.

=cut

sub bg_shift {
    my ($this) = shift();
    return $this->_get_bg_shift(${$this->{obj}});
}

=head1 texture_mask

Gets a mask that tells which bits are used to represent the 
texture (font) index.  See the LibGGI docs.

=cut

sub texture_mask {
    my ($this) = shift();
    return $this->_get_texture_mask(${$this->{obj}});
}

=head1 texture_shift

Gets a shift value that can be used to generate the bits
to represent a texture (font) index.  See the LibGGI docs.

=cut

sub texture_shift {
    my ($this) = shift();
    return $this->_get_texture_shift(${$this->{obj}});
}


=head1 flags

Gets the flag value corresponding to the PF_* constants.

=cut

sub flags {
    my ($this) = shift();
    return $this->_get_flags(${$this->{obj}});
}

=head1 meaning

Gets the "pixel meaning" for the requested bit position;  These
correspond to the BM_* macros.

=cut

sub meaning {
    my ($this) = shift();
    return $this->_get_meaning(${$this->{obj}}, shift());
}

=head1 format

Gets the "stdformat" value.  If this value is the same as another
GGI::Pixel's, but not 0, data can be copied from one to another without any
translation.  For advanced use only.

=cut

sub format {
    my ($this) = shift();
    return $this->_get_stdformat(${$this->{obj}});
}

=head1 buffer

Gets a raw pointer to the buffer where the pixels are stored.
For advanced usage only.

=cut

sub buffer {
    my($this) = shift();
    return $this->{buf};
}

=head2 size
    
Without an argument, returns the number of pixels that will fit in the
allocated buffer associated with the GGI::Pixel argument.  With an
argument, reallocs the buffer to fit the given number of pixels.

=cut

sub size {
    my($this) = shift();
    return $this->{size} unless (@_ + 0);
    my($size) = shift();
    $size = 0 unless $size > 0;
    if ($size == 0) {
	$this->_free_buffer($this->{buf});
	$this->{buf} = undef;
	return(0);
    }
    unless (defined($this->{buf})) {
	$this->{buf} = $this->_malloc(${$this->{obj}}, $size);
        return($this->{size} = 0) unless $this->{buf};
        return($this->{size} = $size);
    }
    $this->{buf} = $this->_realloc($this->{buf}, 
				   ${$this->{obj}}, ${$this->{obj}}, 
                                   $this->{size}, $size);
    return($size) if $this->{size} == $size;
    $this->{size} = $size;
    return($size);
}

sub AUTOLOAD {
    my $constname;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    my $val = constant($constname, @_ ? $_[0] : 0);
    if ($! != 0) {
	if ($! =~ /Invalid/) {
	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
	    goto &AutoLoader::AUTOLOAD;
	}
	else {
		croak "Your vendor has not defined GGI::Pixel macro $constname";
	}
    }
    eval "sub $AUTOLOAD { $val }";
    goto &$AUTOLOAD;
}

bootstrap GGI::Pixel $VERSION;

# Preloaded methods 

1;
__END__

=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GGI::(3pm), perl(1).

=cut
